import os
import configparser
from pathlib import Path
from werkzeug.middleware.proxy_fix import ProxyFix
from .. import _module_path, logger

default = {
    'MAIL': {
        'CRYPT': 'SSL/STARTLS'
    }
}

config = configparser.ConfigParser()
config.read_dict(default)
paths = [_module_path, Path.home()/".config"]
if 'FLASCHENGEIST_CONF' in os.environ:
    paths.append(Path(os.environ.get("FLASCHENGEIST_CONF")))
for loc in paths:
    try:
        with (loc/"flaschengeist.cfg").open() as source:
            config.read_file(source)
    except IOError:
        pass

# Always enable this builtin plugins!
config.read_dict({
    'auth': {
        'enabled': True
    }
})


def configure_app(app):
    if not config.has_option('FLASCHENGEIST', 'SECRET_KEY'):
        logger.warn('No secret key was configured, please configure one for production systems!')
    app.config['SECRET_KEY'] = config.get('FLASCHENGEIST', 'SECRET_KEY', fallback='0a657b97ef546da90b2db91862ad4e29')

    app.config['SQLALCHEMY_DATABASE_URI'] = 'mysql://{user}:{passwd}@{host}/{database}'.format(
        user=config['DATABASE']['user'],
        passwd=config['DATABASE']['passwd'],
        host=config['DATABASE']['host'],
        database=config['DATABASE']['database']
    )
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False

    if config.has_option("FLASCHENGEIST", "ROOT"):
        app.config["APPLICATION_ROOT"] = config["FLASCHENGEIST"]["ROOT"]
    if config.getboolean("FLASCHENGEIST", "PROXY", fallback=False):
        app.wsgi_app = ProxyFix(app.wsgi_app, x_proto=1, x_host=1)
